#ifndef PDF_H
#define PDF_H

#include "PDFPage.h"
#include "PDFOutline.h"
#include "mustd.h"

class PDFPage;
class PDFOutline;

char mytolower(char in);

/// PDF Class
/// This class does most of the work for the library
/// Pass the constructor the path to the file, and it constructs
/// the underlying needed MuPDF structures
/// Initialize must be called after "SetPassword" 
///
class PDF
{
public:
    ///PDF() Constructor
    /// filename is the path of the pdf file to open
    PDF(char *filename);
    ~PDF();

    /// Initialize()
    /// Initializes PDF File structures that may need password
    /// So instantiate, check password protection, set password, then initialize
    int Initialize();

    //Returns wether initialization had an error
    bool HasInitError() {return mInitError;}

    /// IsPasswordProtected() returns true if file requires password, otherwise returns false
    bool IsPasswordProtected();
    /// Sets the password key, returns 0 if successful
    int SetPassword(char *password);

    /// GetPageCount() Returns the number of pages in PDF file
    int GetPageCount();
    /// NextPage() Sets the "Current Page" to the next page, returns 0 on success <0 on failures
    int NextPage();
    /// PreviousPage() Sets the "Current Page" to the previous page, 
    /// returns 0 on success <0 on failures
    int PreviousPage();
    /// JumpToPage() takes the page number to jump to, sets current page to that page
    /// returns 0 on success, <0 on failures
    int JumpToPage(int pageNumber);
    /// GetCurrentPageNumber() Returns the page number of CurrentPage
    int GetCurrentPageNumber();

    /// LoadPage(), loads page with pageNumber and returns the instantiated page object
    PDFPage *LoadPage(int pageNumber);
    /// GetCurrentPage(), Returns a pointer to the CurrentPage object.
    PDFPage *GetCurrentPage();
    /// SetCurrentPage(), sets currentpage to inPage.  Retuns 0 on success < 0 on failure.
    int SetCurrentPage(PDFPage *inPage);
    /// Free's the memory associated with CurrentPage.
    void FreeCurrentPage();

    /// Checks if there is a link at page coordinates x,y and returns the PageNumber or 0 on
    /// success (depending on getPageNumber == true).
    int HasLinkAtLocation(int x, int y, char *, bool getPageNumber);

    /// Returns a PDFOutline Object associated with this PDF file
    PDFOutline *GetOutline();

    /// Dumps the pdf to text
    void DumpAllPagesToText();
    /// Interface for search, needle is what we are looking for, needlelength is strlen of needle
    /// lowpage, highpage are the range of pages to search and 
    /// RecordResult is the call back when we find a result.
    /// RecordResult is passed the page number, boundinng box and "context" of the result.
    /// UserData is data to be passede back to RecordResult for convenience.
    void CaseInsensitiveTextSearch(char *needle, int needlelength, int lowpage, int highpage, 
				   void (*RecordResult)(int page, int x0, int y0, int x1, int y1, int contextlen, char *context, void *), void *userdata);


protected:
    int  GetLinkPageNumber(pdf_link *link);
    void die(fz_error *error);
    void die(char *);

    bool mInitError;
    pdf_xref *mSrc;
    pdf_pagetree *mSrcPages; 
    PDFPage *mCurrentPage;
    PDFOutline *mOutline;
    friend class PDFPage;
    friend class PDFOutline;
};
#endif
